#
#  geneLocusCounts.py
#  ENRAGE
#

try:
    import psyco
    psyco.full()
except:
    pass

import sys
print '%s: version 2.5' % sys.argv[0]
from commoncode import *

if len(sys.argv) < 4:
    print 'usage: python %s genome hitfile outfilename [upstream] [downstream] [-noCDS] [-spanTSS] [-regions acceptfile]' % sys.argv[0]
    print '\twhere upstream and downstream are in bp and and optional'
    print '\tusing noCDS requires either upstream or downstream (but not both)'
    print '\tto be nonzero\n'
    sys.exit(1)

from cistematic.genomes import Genome
from cistematic.core.geneinfo import geneinfoDB

genome = sys.argv[1]
hitfile =  sys.argv[2]
readlen = readSize(hitfile, verbose=True)
outfilename = sys.argv[3]

upstream = 0
downstream = 0
useCDS = True
spanTSS = False
acceptfile = ''
acceptDict = {}
if len(sys.argv) > 4:
    try:
        upstream = int(sys.argv[4])
    except:
        pass

if len(sys.argv) > 5:
    try:
        downstream = int(sys.argv[5])
    except:
        pass

if '-noCDS' in sys.argv:
    useCDS = False

if '-spanTSS' in sys.argv:
    spanTSS = True

if '-regions' in sys.argv:
    acceptfile = sys.argv[-1]
    acceptDict = getMergedRegions(acceptfile, maxDist=0, keepLabel=True, verbose=True)

print 'upstream = %d downstream = %d useCDS = %s spanTSS = %s' % (upstream, downstream, useCDS, spanTSS)

hitDict = getReadDict(hitfile, verbose=True)

hg = Genome(genome)
idb = geneinfoDB(cache=True)

gidCount = {}
gidLen = {}
geneinfoDict = idb.getallGeneInfo(genome)
locusByChromDict = getLocusByChromDict(hg, upstream, downstream, useCDS, acceptDict, upstreamSpanTSS = spanTSS)

gidList = hg.allGIDs()
gidList.sort()
for chrom in acceptDict:
    for (label, start, stop, length) in acceptDict[chrom]:
        if label not in gidList:
            gidList.append(label)
for gid in gidList:
    gidCount[gid] = 0

index = 0
for chrom in hitDict:
    if chrom not in locusByChromDict:
        continue
    for (start, stop, gid, glen) in locusByChromDict[chrom]:
        gidLen[gid] = glen
    print '\n' + chrom + ' ',
    startLocus = 0
    for (tagStart, sense) in hitDict[chrom]:
        index += 1
        if index % 100000 == 0:
            print 'read %d ' % index,
        stopPoint = tagStart + readlen
        if startLocus < 0:
            startLocus = 0
        for (start, stop, gid, glen) in locusByChromDict[chrom][startLocus:]:
            if tagStart > stop:
                startLocus += 1
                continue
            if start > stopPoint:
                startLocus -= 10
                break
            if start <= tagStart <= stop:
                try:
                    gidCount[gid] += 1
                except:
                    continue
                stopPoint = stop

outfile = open(outfilename,'w')

index /= 1000000.
for gid in gidList:
    if 'FAR' not in gid:
        symbol = 'LOC' + gid
        geneinfo = ''
        try:
            geneinfo = geneinfoDict[gid]
            symbol = geneinfo[0][0]
        except:
            #print 'problem with %s' % (gid)
            pass
    else:
        symbol = gid
    if gid in gidCount and gid in gidLen:
        rpm  = gidCount[gid] / index
        rpkm = 1000. * rpm / gidLen[gid]
        outfile.write('%s\t%s\t%d\t%d\t%2.2f\t%2.2f\n' % (gid, symbol, gidCount[gid], gidLen[gid], rpm, rpkm))
outfile.close()

