try:
    import psyco
    psyco.full()
except:
    print 'psyco not running'

import sys

print '%s: version 3.6' % sys.argv[0]

if len(sys.argv) < 5:
    print 'usage: python %s genome hitfile uniqcountfile [acceptfile] outfile [-cache]' % sys.argv[0]
    sys.exit(1)

from commoncode import *
from cistematic.genomes import Genome
from cistematic.core.geneinfo import geneinfoDB
from cistematic.core import chooseDB, cacheGeneDB, uncacheGeneDB

genome = sys.argv[1]
readlen = readSize(sys.argv[2], verbose=True)
hitfile =  sys.argv[2]
countfile = sys.argv[3]
if (len(sys.argv) == 6 and '-cache' not in sys.argv) or len(sys.argv) == 7:
    acceptfile = sys.argv[4]
    outfilename = sys.argv[5]
else:
    acceptfile = 'none'
    outfilename = sys.argv[4]
    acceptDict = {}
hitDict = getMultiReadDict(hitfile, verbose = True)
#readDict = getMultiReadIDDict(hitfile, verbose = True)

if acceptfile != 'none':
    acceptDict = getMergedRegions(acceptfile, maxDist=0, keepLabel=True, verbose=True)

if '-cache' in sys.argv:
    cacheGeneDB(genome)
    hg = Genome(genome, dbFile=chooseDB(genome))
    idb = geneinfoDB(cache=True)
    print '%s cached' % genome
else:
    hg = Genome(genome)
    idb = geneinfoDB()
    
geneinfoDict = idb.getallGeneInfo(genome)
gidCount = {}
gidReadDict = {}

featuresByChromDict = getFeaturesByChromDict(hg, acceptDict)
gidList = hg.allGIDs()

gidList.sort()
for chrom in acceptDict:
    for (label, start, stop, length) in acceptDict[chrom]:
        if label not in gidList:
            gidList.append(label)

for gid in gidList:
    gidCount[gid] = 0
    gidReadDict[gid] = []

uniqueCountDict = {}
gidDict = {}
read2GidDict = {}

uniquecounts = open(countfile)
for line in uniquecounts:
    fields = line.strip().split()
    # add a pseudo-count here to ease calculations below
    uniqueCountDict[fields[0]] = float(fields[-1]) + 1
uniquecounts.close()

#infile = open(infilename)
outfile = open(outfilename,'w')

index = 0
for chrom in hitDict:
    if chrom not in featuresByChromDict:
        continue
    print '\n' + chrom + ' ',
    startFeature = 0
    for (tagStart, tagReadID) in hitDict[chrom]:
        index += 1
        if index % 100000 == 0:
            print 'read %d' % index,
        stopPoint = tagStart + readlen
        if startFeature < 0:
            startFeature = 0
        for (start, stop, gid, sense, ftype) in featuresByChromDict[chrom][startFeature:]:
            if tagStart > stop:
                startFeature += 1
                continue
            if start > stopPoint:
                startFeature -= 100
                break
            if start <= tagStart <= stop:
                if gid in gidCount:
                    gidReadDict[gid].append(tagReadID)
                    if tagReadID not in read2GidDict:
                        read2GidDict[tagReadID] = []
                    if gid not in read2GidDict[tagReadID]:
                        read2GidDict[tagReadID].append(gid)
                    gidCount[gid] += 1
                stopPoint = stop

for gid in gidList:
    if 'FAR' not in gid:
        symbol = 'LOC' + gid
        geneinfo = ''
        try:
            geneinfo = geneinfoDict[gid]
            if genome == 'celegans':
                symbol = geneinfo[0][1]
            else:
                symbol = geneinfo[0][0]
        except:
            #print 'problem with %s' % (gid)
            pass
    else:
        symbol = gid
    tagCount = 0.
    for readID in gidReadDict[gid]:
        try:
            tagValue = uniqueCountDict[gid]
        except:
            tagValue = 1
        tagDenom = 0.
        for aGid in read2GidDict[readID]:
            try:
                tagDenom += uniqueCountDict[aGid]
            except:
                tagDenom += 1
        tagCount += tagValue / tagDenom
    print '%s %s %f' % (gid, symbol, tagCount)
    outfile.write('%s\t%s\t%d\n' % (gid, symbol, tagCount))
#infile.close()
outfile.close()


if '-cache' in sys.argv:
    uncacheGeneDB(genome)
