import sys

try:
    import psyco
    psyco.full()
except:
    print 'psyco not running'
from cistematic.core import complement
from cistematic.core.motif import Motif
from cistematic.genomes import Genome
from commoncode import *

print 'version 1.2'
if len(sys.argv) < 5:
    print 'usage: python %s genome ChIPbedfile regionfile siteOutfile [-plot badregionPlot.png]' % sys.argv[0]
    sys.exit(1)

doPlot = False
if '-plot' in sys.argv:
    import matplotlib
    matplotlib.use('Agg')
    
    from pylab import *
    doPlot = True
    plotname = sys.argv[sys.argv.index('-plot') + 1]

genome = sys.argv[1]
chipfilename = sys.argv[2]
#pvalfilename = 'pvalueunamp.txt'
infilename = sys.argv[3]
outfilename = sys.argv[4]
maxPvalue = 0.0001

mot = Motif('',motifFile = 'NRSE2.mot')
motL = Motif('',motifFile = 'NRSE2left.mot')
motR = Motif('',motifFile = 'NRSE2right.mot')
bestScore = mot.bestConsensusScore()

hg = Genome(genome)
#genomesize = getGenomesize()

regions = getMergedRegions(infilename, 500)
hitDict = getReadDict(chipfilename)

outfile = open(outfilename,'w')

countList = []
#pvalfile = open(pvalfilename)
#for line in pvalfile:
#	fields = line.split()
#	if len(fields) > 1:
#		countList.append(int(fields[1]))

posList = []

index = 0
regionList = []

for chrom in regions:
	if 'rand' in chrom or 'M' in chrom:
		continue
	for (start, stop, length) in regions[chrom]:
		regionList.append((chrom, start, length))
	
notFoundIndex = 0
for (chrom, start, length) in regionList:
	seq = hg.sequence(chrom, start, length)
	(topPos, numHits, smoothArray, numPlus) = findPeak(hitDict[chrom], start, length)
	if len(topPos) == 0:
		print 'topPos error'
	peakpos = topPos[0]
	found = []
	lefts = motL.locateMotif(seq, 70)
	rights = motR.locateMotif(seq, 70)
	allhalfs = [(v0, v1, 'L') for (v0, v1) in lefts] + [(v0, v1, 'R') for (v0, v1) in rights]
	allhalfs.sort()
	
	# look for canonicals and non-canonicals
	if len(allhalfs) > 1:
		(firstpos, firstsense, firsttype) = allhalfs[0]
		for (secondpos, secondsense, secondtype) in allhalfs[1:]:
			dist = secondpos - firstpos
			if firstsense == secondsense and dist in [10, 11, 16, 17, 18, 19]:
				if (firsttype == 'L' and secondtype == 'R' and secondsense == 'F'):
					found.append((start + firstpos, firstpos - peakpos + (dist + 10)/2, dist))
				if (firsttype == 'R' and secondtype == 'L' and secondsense == 'R'):
					found.append((start + firstpos, firstpos  - peakpos + (dist + 10)/2, dist))
			firstpos = secondpos
			firstsense = secondsense
			firsttype = secondtype
		
	# did we miss any 70%+ matches ?
	matches = mot.locateMotif(seq, 70)
	for (pos, sense) in matches:
		alreadyFound = False
		for (fpos, fpeakdist, fdist) in found:
			if pos + start == fpos:
				alreadyFound = True
		if not alreadyFound:
			found.append((start + pos, pos - peakpos + 10, 11))
	
	# we'll now accept half-sites within 100 bp of peak
	if len(found) == 0:
		bestone = -1
		bestdist = 101
		index = 0
		for (pos, sense, type) in allhalfs:
			if abs(pos - peakpos) < bestdist:
				bestdist = abs(pos - peakpos)
				bestone = index
			index += 1
		if bestdist < 101:
			found.append((start + allhalfs[bestone][0], allhalfs[bestone][0] + 5 - peakpos, 0))
	foundValue = False
	for (foundpos, posdist, dist) in found:
		# get a score for 21-mer, report
		seq = hg.sequence(chrom, foundpos, 21)
		# get a p-value
		expected = 0
		midpoint = foundpos + 9 + dist
		tagcount = 0
		for (tstart, sense) in hitDict[chrom]:
			if abs(tstart - midpoint) < 200:
				tagcount += 1
			if tagcount > 199:
				tagcount = 199
				break
		#for mockcount in countList[tagcount:]:
		#	expected += mockcount
		#pval = expected / genomesize
		# reject if pvalue is not small enough
		#if pval > maxPvalue:
		#	continue
		pval = 0.01
		foundValue = True
		(front, back) = mot.scoreMotif(seq)
		sense = '+'
		if front > back:
			score = int(100 * front / bestScore)
                        theseq = hg.sequence(chrom, foundpos, 30)
		else:
			score = int(100 * back / bestScore)
                        theseq = complement(hg.sequence(chrom, foundpos - 9, 30))
			sense = '-'
			foundpos + 1
		outfile.write('chr%s:%d-%d\t%d\t%d\t%d\t%d\t%3.3g\tchr%s:%d-%d\t%s\t%s\n' % (chrom, foundpos, foundpos + 9 + dist, score, numHits, posdist, dist, pval, chrom, start, start + length, sense, theseq))
	
	# we didn't find a site - draw region
	if not foundValue and doPlot:
		print 'no predictions for %s:%d-%d %d' % (chrom, start, start + length, numHits)
		drawarray = [val + notFoundIndex for val in smoothArray]
		drawpos = [drawarray[val] for val in topPos]
		plot(drawarray,'b')
		plot(topPos, drawpos,'r.')
		goodmatches = mot.locateMotif(seq, 75)
		if len(goodmatches) > 0:
			print topPos
			print goodmatches
			drawgood = []
			drawgoody = []
			for (mstart, sense) in goodmatches:
				drawgood.append(mstart)
				drawgoody.append(drawarray[mstart])
			plot(drawgood, drawgoody, 'g.')
		notFoundIndex -= 30
outfile.close()
if doPlot:
    savefig(plotname)
