import sys

print '%s: version 4.6' % sys.argv[0]

try:
    import psyco
    psyco.full()
except:
    print 'psyco not running'

from commoncode import *

if len(sys.argv) < 4:
    print 'usage: python %s name bedfile outfilename [-normalize] [-color xx,yy,zz] [-chrom chromID] [-split] [-listfile filename] [-listprefix prefix] [-group groupName] [-startPriority prefix] [-skiprandom]' % sys.argv[0]
    sys.exit(1)

name = sys.argv[1]
readlen = readSize(sys.argv[2], verbose=True)
hitfilename = sys.argv[2]
outfilename = sys.argv[3]
colorString = ''
doNormalize = False
doSplit = False
doList = False
listPrefix = ''
groupName=''
startPriority=0.01
priorityIncrement = 0.01
skipRandom = False

if '-normalize' in sys.argv:
    doNormalize = True

if '-split' in sys.argv:
    doSplit = True

if '-color' in sys.argv:
    colorField = sys.argv.index('-color') + 1
    try:
        colorString = ' color=' + sys.argv[colorField]
    except:
        pass

if '-listfile' in sys.argv:
    listfilename = sys.argv[sys.argv.index('-listfile') + 1]
    doList = True
if '-listprefix' in sys.argv:
    listPrefix = sys.argv[sys.argv.index('-listprefix') + 1]
    
chromLimit = False
if '-chrom' in sys.argv:
    chromField = sys.argv.index('-chrom') + 1
    chromLimit = True
    limitChrom = sys.argv[chromField]

if '-group' in sys.argv:
    groupName = 'group=' + sys.argv[sys.argv.index('-group') + 1]

if '-startPriority' in sys.argv:
    startPriority = float(sys.argv[sys.argv.index('-startPriority') + 1])

if '-skiprandom' in sys.argv:
    skipRandom = True

if doNormalize:
    normalizeBy = datasetSize(hitfilename) / 1000000.
hitDict = getReadDict(hitfilename, verbose=True, noSense=True, fullChrom=True)

if doList:
    listfile = open(listfilename, 'w')

priority = startPriority    
if not doSplit:
    outfile = open(outfilename,'w')
    if doList:
        listfile.write(listPrefix + outfilename + '\n')
    outfile.write('track type=wiggle_0 name="%s" %s priority=%.3f visibility=full%s\n' % (name, groupName, priority, colorString)) 
chromList = hitDict.keys()
chromList.sort()
for chrom in chromList:
    if chromLimit and chrom != limitChrom:
        continue
    if skipRandom and 'random' in chrom:
        continue
    print chrom
    if doSplit:
        outfile = open(outfilename + '.' + chrom,'w')
        if doList:
            listfile.write(listPrefix + outfilename + '.' + chrom + '\n')
        outfile.write('track type=wiggle_0 name="%s %s" %s priority=%.3f visibility=full%s\n' % (name, chrom, groupName, priority, colorString))   
        priority += priorityIncrement  
    lastNT = hitDict[chrom][-1] + readlen
    chromModel = lastNT * [0]
    for start in hitDict[chrom]:
        for index in range(readlen):
            currentpos = start + index
            chromModel[currentpos] += 1
    previousVal = 0
    previousStart = 1
    for index in xrange(lastNT):
        currentVal = chromModel[index]
        if currentVal == previousVal:
            continue
        if currentVal != previousVal:
            if previousVal > 0:
                lastpos = index
                #lastpos = index -1
                #if lastpos == previousStart:
                #	lastpos += 1
                if doNormalize:
                    outline = '%s %d %d %.2f\n' % (chrom, previousStart, lastpos, previousVal/normalizeBy)
                else:
                    outline = '%s %d %d %d\n' % (chrom, previousStart, lastpos, previousVal)
                outfile.write(outline)
            previousVal = currentVal
            previousStart = index
    if doSplit:
        outfile.close()
if not doSplit:
    outfile.close()
if doList:
    listfile.close()

