try:
    import psyco
    psyco.full()
except:
    pass

import sys
print '%s: version 4.8' % sys.argv[0]
if len(sys.argv) < 6:
    print 'usage: python %s genome uniqbedfile uniqcountfile splicecountfile outfile [candidatefile acceptfile] [-gidField fieldID] [-maxLength kblength] [-cache]\n' % sys.argv[0]
    print "\twhere splicecountfile can be set to 'none' to not count splices\n"
    sys.exit(1)

from commoncode import *
from cistematic.genomes import Genome
from cistematic.core.geneinfo import geneinfoDB
from cistematic.core import chooseDB, cacheGeneDB, uncacheGeneDB

genome = sys.argv[1]
uniquebedfilename = sys.argv[2]
uniquecountfile = open(sys.argv[3])

dosplicecount = False
if 'none' not in sys.argv:
    dosplicecount = True
    splicecountfile = open(sys.argv[4])

candidateLines = []
if len(sys.argv) > 6:
    try:
        candidatefile = open(sys.argv[6])
        candidateLines = candidatefile.readlines()
        candidatefile.close()
        acceptedfile = open(sys.argv[7],'w')
    except:
        pass

fieldID = 1
if '-gidField' in sys.argv:
    gidField = sys.argv.index('-gidField') + 1
    try:
        fieldID = int(sys.argv[gidField])
    except:
        print "couldn't process gidField"
        pass

maxLength = 1000000000.
if '-maxLength' in sys.argv:
    maxField = sys.argv.index('-maxLength') + 1
    try:
        maxLength = float(sys.argv[maxField])
    except:
        print "couldn't process maxLength"
        pass

if '-cache' in sys.argv:
    cacheGeneDB(genome)
    hg = Genome(genome, dbFile=chooseDB(genome))
    idb = geneinfoDB(cache=True)
    print '%s cached' % genome
else:
    hg = Genome(genome)
    idb = geneinfoDB()
    
bedcount = bedcount = datasetSize(uniquebedfilename, verbose=True)
splicecount = 0
countDict = {}
geneList = []
farList = []
candidateDict = {}


gidDict = {}
geneinfoDict = idb.getallGeneInfo(genome)
symbolToGidDict = {}

for gid in geneinfoDict:
    if genome != 'celegans':
        symbol = geneinfoDict[gid][0][0].strip()
        symbolToGidDict[symbol] = gid
    else:
        symbol = geneinfoDict[gid][0][0].strip()
        symbolToGidDict[symbol] = geneinfoDict[gid][0][1].strip()
featuresDict = hg.getallGeneFeatures()
print 'got featuresDict'

outfile = open(sys.argv[5],'w')

for line in uniquecountfile:
    fields = line.strip().split()
    countDict[fields[fieldID]] = float(fields[-1])
    geneList.append(fields[fieldID])
uniquecountfile.close()

if dosplicecount:
    for line in splicecountfile:
        fields = line.strip().split()
        try:
            countDict[fields[fieldID]] += float(fields[-1])
        except:
            print fields
            continue
        splicecount += float(fields[-1])
    splicecountfile.close()

for line in candidateLines:
    fields = line.strip().split()
    if fields[0] not in geneList:
        if fields[0] not in farList:
            farList.append(fields[0])
        if fields[0] not in countDict:
            countDict[fields[0]] = 0
        countDict[fields[0]] += float(fields[-1])
    if fields[0] not in candidateDict:
        candidateDict[fields[0]] = []
    candidateDict[fields[0]].append((float(fields[-1]), abs(int(fields[-2]) - int(fields[-3])), fields[-4], fields[-3], fields[-2]))

totalCount = (bedcount + splicecount) / 1000000.
for gene in geneList:
    featureList = []
    if 'LOC' in gene:
        gid = gene[3:]
    else:
        try:
            gid = symbolToGidDict[gene]
        except:
            gid = gene
    try:
        featureList = featuresDict[gid]
    except:
        try:
            featureList = featuresDict[gene]
        except:
            print gene, gid
    newfeatureList = []
    geneLength = 0.
    for (ftype, chrom, start, stop, sense) in featureList:
        if (start, stop) not in newfeatureList:
            newfeatureList.append((start, stop))
            geneLength += (abs(start - stop) + 1.) / 1000.
    #rpmk is rpm / geneLength in kb.
    if geneLength < 0.1:
        geneLength = 0.1
    elif geneLength > maxLength:
        geneLength = maxLength
    rpm = countDict[gene] / totalCount
    rpmk = rpm / geneLength
    if gene in candidateDict:
        for (cCount, cLength, chrom, cStart, cStop) in candidateDict[gene]:
            cratio = cCount / (cLength / 1000.)
            cratio = cratio / totalCount
            if 10. * cratio < rpmk:
                continue
            countDict[gene] += cCount
            geneLength += cLength / 1000.
            acceptedfile.write('%s\t%s\t%s\t%s\t%.2f\t%d\t%s\n' % (gid, chrom, cStart, cStop, cratio, cLength, gene)) 
    rpm = countDict[gene] / totalCount
    rpmk = rpm / geneLength
    outfile.write('%s\t%s\t%.4f\t%.2f\n' %  (gid, gene, geneLength, rpmk))

for gene in farList:
    geneLength = 0
    for (cCount, cLength, chrom, cStart, cStop) in candidateDict[gene]:
        geneLength += cLength / 1000.
    if geneLength < 0.1:
        continue
    for (cCount, cLength, chrom, cStart, cStop) in candidateDict[gene]:
        cratio = cCount / (cLength / 1000.)
        cratio = cratio / totalCount
        acceptedfile.write('%s\t%s\t%s\t%s\t%.2f\t%d\t%s\n' % (gene, chrom, cStart, cStop, cratio, cLength, gene)) 
    rpm = countDict[gene] / totalCount
    rpmk = rpm / geneLength
    outfile.write('%s\t%s\t%.4f\t%.2f\n' %  (gene, gene, geneLength, rpmk))
outfile.close()
try:
    acceptedfile.close()
except:
    pass

if '-cache' in sys.argv:
    uncacheGeneDB(genome)
