try:
	import psyco
	psyco.full()
except:
	pass

import sys
print '%s: version 2.9' % sys.argv[0]
if len(sys.argv) < 8:
	print 'usage: python %s genome uniqbedfile splicebedfile multibedfile expandedRPKMfile multicountfile outfile [-multifraction] [-multifold] [-minrpkm minThreshold] [-withGID]\n' % sys.argv[0]
	sys.exit(1)

from cistematic.core.geneinfo import geneinfoDB
from commoncode import *

genome = sys.argv[1]
uniquebedfilename = sys.argv[2]
splicebedfilename = sys.argv[3]
multibedfilename = sys.argv[4]
expandedRPKMfile = open(sys.argv[5])
multicountfile = open(sys.argv[6])

writeGID = False
reportFraction = False
reportFold = False 
minThreshold = 0.

if '-withGID' in sys.argv:
    writeGID = True
    
if '-multifraction' in sys.argv:
    reportFraction = True
    print 'reporting fraction contribution of multireads'
elif '-multifold' in sys.argv:
    reportFold = True
    print 'reporting fold contribution of multireads'

if '-minrpkm' in sys.argv:
    minThreshold = float(sys.argv[sys.argv.index('-minrpkm') + 1])
    print 'returning only genes with > %.2f RPKM' % minThreshold
    
bedcount = datasetSize(uniquebedfilename, verbose=True)
splicecount = datasetSize(splicebedfilename, verbose=True)
multicount = datasetSize(multibedfilename, verbose=True, countLabels=True)
countDict = {}
multicountDict = {}
lengthDict = {}
geneList = []

uniqspliceCount = (bedcount + splicecount) / 1000000.
totalCount = (bedcount + splicecount + multicount) / 1000000.
idb = geneinfoDB(cache=True)

gidDict = {}
geneinfoDict = idb.getallGeneInfo(genome)
symbolToGidDict = {}

for gid in geneinfoDict:
    if genome != 'celegans':
        symbol = geneinfoDict[gid][0][0].strip()
        symbolToGidDict[symbol] = gid
    else:
        symbol = geneinfoDict[gid][0][0].strip()
        symbolToGidDict[symbol] = geneinfoDict[gid][0][1].strip()

outfile = open(sys.argv[7],'w')

for line in expandedRPKMfile:
    fields = line.strip().split()
    countDict[fields[1]] = float(fields[-1]) * float(fields[-2]) * uniqspliceCount
    lengthDict[fields[1]] = float(fields[-2])
    geneList.append(fields[1])
expandedRPKMfile.close()

for line in multicountfile:
    fields = line.strip().split()
    gene = fields[1]
    if 'LOCFAR' in gene:
        gene = gene[3:]
    if genome == 'celegans' and 'LOC' in gene:
        gene = gene[3:]
    if gene in countDict:
        countDict[gene] += float(fields[-1])
        multicountDict[gene] = float(fields[-1])
multicountfile.close()

index = 0
for gene in geneList:
    if 'LOC' in gene:
        gid = gene[3:]
    elif 'FAR' in gene or genome == 'celegans':
        gid = gene
    else:
        try:
            gid = symbolToGidDict[gene]
        except:
            gid = gene
    #rpkm is rpm / geneLength in kb.
    rpm = countDict[gene] / totalCount
    rpkm = rpm / lengthDict[gene]
    if rpkm < minThreshold:
        continue
    if writeGID:
        outfile.write('%s\t' % gid)
    index += 1
    multirpm = multicountDict[gene] / totalCount
    multirpkm = multirpm / lengthDict[gene]
    if reportFraction or reportFold:
        try:
            if reportFraction:
                multivalue = multirpkm / rpkm
            else:
                if rpm > multirpm:
                    uniqrpkm = (rpm - multirpm) / lengthDict[gene]
                    multivalue = rpkm / uniqrpkm
                elif rpkm > 0.01:
                    multivalue = 100.
                else:
                    multivalue = 1.0
        except:
            multivalue = 0
        outfile.write('%s\t%.3f\t%.2f\t%.2f\n' %  (gene, lengthDict[gene], rpkm, multivalue))
    else:
        outfile.write('%s\t%.3f\t%.2f\n' %  (gene, lengthDict[gene], rpkm))
outfile.close()

print 'returned %d genes' % index